@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 // Unfortunately they don't define such constants so we have to define
 // them here.  If they change in nDPI, they'll need to be updated
 // manually.
+#define ND_FLOW_HOSTNAME    80      // nDPI host_server_name length
 #define ND_FLOW_UA_LEN      512     // User agent length
 #define ND_FLOW_URL_LEN     512     // HTTP URL length
 #define ND_FLOW_SSH_UALEN   48      // SSH user-agent (signature) length
@@ -32,7 +33,6 @@
 #define ND_FLOW_TLS_JA3LEN  33      // TLS JA3 hash length (MD5)
 #define ND_FLOW_DHCPFP_LEN  48      // DHCP fingerprint length
 #define ND_FLOW_DHCPCI_LEN  96      // DHCP class identifier
-#define ND_FLOW_MDNS_ANSLEN 96      // MDNS answer length
 
 // BitTorrent info hash length
 #define ND_FLOW_BTIHASH_LEN     SHA1_DIGEST_LENGTH
@@ -40,6 +40,9 @@
 // SSL certificate fingerprint hash length
 #define ND_FLOW_TLS_HASH_LEN    SHA1_DIGEST_LENGTH
 
+// Extra protocol info text
+#define ND_FLOW_EXTRA_INFO      16
+
 // Capture filename template
 #define ND_FLOW_CAPTURE_TEMPLATE    ND_VOLATILE_STATEDIR "/nd-flow-XXXXXXXX.cap"
 #define ND_FLOW_CAPTURE_SUB_OFFSET  (sizeof(ND_FLOW_CAPTURE_TEMPLATE) - 8 - 4 - 1)
@@ -52,12 +55,10 @@ typedef unordered_map<string, string> nd_flow_kvmap;
 class ndFlow
 {
 public:
-    nd_ifaces::iterator iface;
+    nd_interface::iterator iface;
 
     int16_t dpi_thread_id;
 
-    uint8_t *pkt;
-
     uint8_t ip_version;
     uint8_t ip_protocol;
 
@@ -124,19 +125,25 @@ public:
 
     uint8_t detection_packets;
 
-    ndpi_protocol detected_protocol;
-    char *detected_protocol_name;
+    nd_proto_id_t detected_protocol;
+    nd_app_id_t detected_application;
+
+    const char *detected_protocol_name;
     char *detected_application_name;
 
-    struct ndpi_flow_struct *ndpi_flow;
+    struct {
+        nd_cat_id_t application;
+        nd_cat_id_t protocol;
+        nd_cat_id_t domain;
+    } category;
 
-    struct ndpi_id_struct *id_src;
-    struct ndpi_id_struct *id_dst;
+    struct ndpi_flow_struct *ndpi_flow;
 
     uint8_t digest_lower[SHA1_DIGEST_LENGTH];
     uint8_t digest_mdata[SHA1_DIGEST_LENGTH];
 
-    char host_server_name[ND_MAX_HOSTNAME];
+    char dns_host_name[ND_FLOW_HOSTNAME];
+    char host_server_name[ND_FLOW_HOSTNAME];
 
     union {
         struct {
@@ -157,13 +164,13 @@ public:
         struct {
             uint16_t version;
             uint16_t cipher_suite;
-            char client_sni[ND_FLOW_TLS_CNLEN];
+            char *client_sni, *subject_dn, *issuer_dn;
             char server_cn[ND_FLOW_TLS_CNLEN];
-            char server_organization[ND_FLOW_TLS_ORGLEN];
             char client_ja3[ND_FLOW_TLS_JA3LEN];
             char server_ja3[ND_FLOW_TLS_JA3LEN];
             bool cert_fingerprint_found;
             char cert_fingerprint[ND_FLOW_TLS_HASH_LEN];
+            vector<string> alpn, alpn_server;
         } ssl;
 
         struct {
@@ -174,10 +181,11 @@ public:
             uint8_t info_hash_valid:1;
             char info_hash[ND_FLOW_BTIHASH_LEN];
         } bt;
-
+#if 0
         struct {
-            char answer[ND_FLOW_MDNS_ANSLEN];
-        } mdns;
+            char variant[ND_FLOW_EXTRA_INFO];
+        } mining;
+#endif
     };
 
     struct {
@@ -217,7 +225,7 @@ public:
     // Start of conditional members.  These must be at the end or else access
     // from plugins compiled without various options will have incorrect
     // addresses
-#ifdef _ND_USE_CONNTRACK
+#if defined(_ND_USE_CONNTRACK) && defined(_ND_WITH_CONNTRACK_MDATA)
     uint32_t ct_id;
     uint32_t ct_mark;
 #endif
@@ -226,25 +234,21 @@ public:
     ndNetlinkAddressType upper_type;
 #endif
     struct {
-#ifdef HAVE_ATOMIC
-        atomic<uint8_t> ip_nat;
-        atomic<uint8_t> tcp_fin;
-        atomic<uint8_t> dhc_hit;
-        atomic<uint8_t> detection_complete;
-        atomic<uint8_t> detection_expiring;
-        atomic<uint8_t> detection_expired;
-        atomic<uint8_t> detection_guessed;
-#else
-        uint8_t ip_nat:1;
-        uint8_t tcp_fin:1;
-        uint8_t dhc_hit:1;
-        uint8_t detection_complete:1;
-        uint8_t detection_expiring:1;
-        uint8_t detection_expired:1;
-        uint8_t detection_guessed:1;
-#endif
+        atomic_bool ip_nat;
+        atomic_bool tcp_fin;
+        atomic_bool dhc_hit;
+        atomic_bool detection_init;
+        atomic_bool detection_complete;
+        atomic_bool detection_expiring;
+        atomic_bool detection_expired;
+        atomic_bool detection_guessed;
+        atomic_bool detection_updated;
+        atomic_bool risk_checked;
+        atomic_bool soft_dissector;
     } flags;
 
+    atomic_uint queued;
+
     union {
         struct {
             uint8_t version;
@@ -266,7 +270,12 @@ public:
         } gtp;
     };
 
-    ndFlow(nd_ifaces::iterator iface);
+    vector<nd_risk_id_t> risks;
+    uint16_t ndpi_risk_score;
+    uint16_t ndpi_risk_score_client;
+    uint16_t ndpi_risk_score_server;
+
+    ndFlow(nd_interface::iterator iface);
     ndFlow(const ndFlow &flow);
     virtual ~ndFlow();
 
@@ -281,24 +290,26 @@ public:
 
     void release(void);
 
-    uint16_t master_protocol(void);
+    nd_proto_id_t master_protocol(void) const;
 
-    bool has_dhcp_fingerprint(void);
-    bool has_dhcp_class_ident(void);
-    bool has_http_user_agent(void);
-    bool has_http_url(void);
-    bool has_ssh_client_agent(void);
-    bool has_ssh_server_agent(void);
-    bool has_ssl_client_sni(void);
-    bool has_ssl_server_cn(void);
-    bool has_ssl_server_organization(void);
-    bool has_ssl_client_ja3(void);
-    bool has_ssl_server_ja3(void);
-    bool has_bt_info_hash(void);
-    bool has_mdns_answer(void);
-    bool has_ssdp_headers(void);
-
-    void print(void);
+    bool has_dhcp_fingerprint(void) const;
+    bool has_dhcp_class_ident(void) const;
+    bool has_http_user_agent(void) const;
+    bool has_http_url(void) const;
+    bool has_ssh_client_agent(void) const;
+    bool has_ssh_server_agent(void) const;
+    bool has_ssl_client_sni(void) const;
+    bool has_ssl_server_cn(void) const;
+    bool has_ssl_issuer_dn(void) const;
+    bool has_ssl_subject_dn(void) const;
+    bool has_ssl_client_ja3(void) const;
+    bool has_ssl_server_ja3(void) const;
+    bool has_bt_info_hash(void) const;
+    bool has_ssdp_headers(void) const;
+#if 0
+    bool has_mining_variant(void) const;
+#endif
+    void print(void) const;
 
     void update_lower_maps(void);
     void get_lower_map(
