@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -46,11 +46,16 @@ void nd_print_binary(uint32_t byte);
 
 void nd_print_number(ostringstream &os, uint64_t value, bool units_binary = true);
 
+void nd_ltrim(string &s, unsigned char c = 0);
+void nd_rtrim(string &s, unsigned char c = 0);
+void nd_trim(string &s, unsigned char c = 0);
+
 int nd_sha1_file(const string &filename, uint8_t *digest);
 void nd_sha1_to_string(const uint8_t *digest_bin, string &digest_str);
 
 bool nd_string_to_mac(const string &src, uint8_t *mac);
 sa_family_t nd_string_to_ip(const string &src, sockaddr_storage *ip);
+bool nd_ip_to_string(sa_family_t af, const void *addr, string &dst);
 bool nd_ip_to_string(const sockaddr_storage &ip, string &dst);
 
 void nd_iface_name(const string &iface, string &result);
@@ -72,6 +77,8 @@ void nd_generate_uuid(string &uuid);
 
 string nd_get_version_and_features(void);
 
+bool nd_parse_app_tag(const string &tag, unsigned &id, string &name);
+
 #ifdef _ND_USE_WATCHDOGS
 int nd_touch(const string &filename);
 #endif
@@ -103,5 +110,31 @@ int nd_functions_exec(const string &func, string &output);
 
 void nd_os_detect(string &os);
 
+#define _ND_LOG_FILE_STAMP      "%Y%m%d-%H%M%S"
+#define _ND_LOG_FILE_STAMP_SIZE sizeof("YYYYMMDD-HHMMSS")
+
+class ndLogDirectory
+{
+public:
+    ndLogDirectory(const string &path, const string &prefix, const string &suffix);
+    virtual ~ndLogDirectory();
+
+    FILE *Open(void);
+    void Close(void);
+    void Discard(void);
+
+protected:
+    string path;
+    string prefix;
+    string suffix;
+
+    FILE *hf_cur;
+    string filename;
+};
+
+void nd_regex_error(const regex_error &e, string &error);
+
+bool nd_scan_dotd(const string &path, vector<string> &files);
+
 #endif // _ND_UTIL_H
 // vi: expandtab shiftwidth=4 softtabstop=4 tabstop=4
