@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,6 +19,10 @@
 
 #include "nd-json-response-code.h"
 
+#ifndef AF_LINK
+#define AF_LINK AF_PACKET
+#endif
+
 #ifndef ETH_ALEN
 #include <net/ethernet.h>
 #if !defined(ETH_ALEN) && defined(ETHER_ADDR_LEN)
@@ -73,8 +77,7 @@
 #define ND_MAX_PKT_QUEUE_KB     8192    // Maximum packet queue size in kB
 #define ND_PKTQ_FLUSH_DIVISOR   10      // Divisor of PKT_QUEUE_KB packets to flush.
 
-#define ND_MAX_TCP_PKTS         10      // Maximum number of TCP packets to process.
-#define ND_MAX_UDP_PKTS         8       // Maximum number of UDP packets to process.
+#define ND_MAX_DETECTION_PKTS   32      // Maximum number of packets to process.
 
 #ifndef ND_VOLATILE_STATEDIR
 #define ND_VOLATILE_STATEDIR    "/var/run/netifyd"
@@ -112,7 +115,7 @@
 #define ND_JSON_DATA_CHUNKSIZ   4096
 #define ND_JSON_INDENT          4
 
-#define ND_PCAP_SNAPLEN         1536    // Capture snap length
+#define ND_PCAP_SNAPLEN         65535   // Capture snap length
 #define ND_PCAP_READ_TIMEOUT    500     // Milliseconds
 
 #ifndef ND_URL_SINK
@@ -156,26 +159,36 @@
 #define ND_SOCKET_PATH_GROUP    "root"
 
 #ifndef PACKAGE_URL
-#define PACKAGE_URL             "http://www.egloo.ca/"
+#define PACKAGE_URL             "https://www.netify.ai/"
 #endif
 
-#define ND_CONF_SINK_BASE       "netify-sink.conf"
-#define ND_CONF_SINK_PATH       ND_PERSISTENT_STATEDIR "/" ND_CONF_SINK_BASE
+#define ND_CONF_APP_BASE        "netify-apps.conf"
+#define ND_CONF_APP_PATH        ND_PERSISTENT_STATEDIR "/" ND_CONF_APP_BASE
+
+#define ND_CONF_CAT_BASE        "netify-categories.json"
+#define ND_CONF_CAT_PATH        ND_PERSISTENT_STATEDIR "/" ND_CONF_CAT_BASE
+
+#define ND_CONF_LEGACY_BASE     "netify-sink.conf"
+#define ND_CONF_LEGACY_PATH     ND_PERSISTENT_STATEDIR "/" ND_CONF_LEGACY_BASE
 
 #define ND_STR_ETHALEN          (ETH_ALEN * 2 + ETH_ALEN - 1)
 
 #define ND_PRIVATE_IPV4         "127.255.0."
 #define ND_PRIVATE_IPV6         "fe:80::ffff:7fff:"
 
+#define ND_API_UPDATE_TTL       (3600 * 24)
+#define ND_API_UPDATE_URL       "https://api.netify.ai/api/v1"
+#define ND_API_VENDOR           "EG"
+
 #include "nd-sha1.h"
 
 typedef unordered_map<string, vector<string> > nd_device_addrs;
-typedef map<string, pair<pthread_mutex_t *, nd_device_addrs *> > nd_devices;
-typedef unordered_map<string, string> nd_device_ethers;
-typedef vector<pair<bool, string> > nd_ifaces;
+typedef map<string, pair<pthread_mutex_t *, nd_device_addrs *> > nd_device;
+typedef unordered_map<string, string> nd_device_ether;
+typedef vector<pair<bool, string> > nd_interface;
 typedef vector<pair<string, string> > nd_device_addr;
 typedef map<string, string> nd_device_filter;
-typedef map<string, string> nd_device_netlink;
+typedef map<string, string> nd_netlink_device;
 typedef map<string, string> nd_inotify_watch;
 #ifdef _ND_USE_PLUGINS
 class ndPluginLoader;
@@ -183,129 +196,6 @@ typedef map<string, ndPluginLoader *> nd_plugins;
 #endif
 typedef pair<struct sockaddr_storage, struct sockaddr_storage> nd_private_addr;
 
-enum nd_dhc_save {
-    ndDHC_DISABLED = 0,
-    ndDHC_PERSISTENT = 1,
-    ndDHC_VOLATILE = 2
-};
-
-enum nd_fhc_save {
-    ndFHC_DISABLED = 0,
-    ndFHC_PERSISTENT = 1,
-    ndFHC_VOLATILE = 2
-};
-
-enum nd_global_flags {
-    ndGF_DEBUG = 0x1,
-    ndGF_DEBUG_UPLOAD = 0x2,
-    ndGF_DEBUG_WITH_ETHERS = 0x4,
-    ndGF_QUIET = 0x8,
-    ndGF_OVERRIDE_SINK_CONFIG = 0x10,
-    ndGF_CAPTURE_UNKNOWN_FLOWS = 0x20,
-    ndGF_PRIVATE_EXTADDR = 0x40,
-    ndGF_SSL_USE_TLSv1 = 0x80,
-    ndGF_SSL_VERIFY = 0x100,
-    ndGF_USE_CONNTRACK = 0x200,
-    ndGF_USE_NETLINK = 0x400,
-    ndGF_FREE_0x800 = 0x800,
-    ndGF_USE_SINK = 0x1000,
-    ndGF_USE_DHC = 0x2000,
-    ndGF_USE_FHC = 0x4000,
-    ndGF_EXPORT_JSON = 0x8000,
-    ndGF_VERBOSE = 0x10000,
-    ndGF_REPLAY_DELAY = 0x20000,
-    ndGF_REMAIN_IN_FOREGROUND = 0x40000,
-    ndGF_FLOW_DUMP_ESTABLISHED = 0x80000,
-    ndGF_FLOW_DUMP_UNKNOWN = 0x100000,
-    ndGF_UPLOAD_ENABLED = 0x200000,
-    ndGF_UPLOAD_NAT_FLOWS = 0x400000,
-    ndGF_WAIT_FOR_CLIENT = 0x800000
-};
-
-#define ND_DEBUG (nd_config.flags & ndGF_DEBUG)
-#define ND_DEBUG_UPLOAD (nd_config.flags & ndGF_DEBUG_UPLOAD)
-#define ND_DEBUG_WITH_ETHERS (nd_config.flags & ndGF_DEBUG_WITH_ETHERS)
-#define ND_QUIET (nd_config.flags & ndGF_QUIET)
-#define ND_OVERRIDE_SINK_CONFIG (nd_config.flags & ndGF_OVERRIDE_SINK_CONFIG)
-#define ND_CAPTURE_UNKNOWN_FLOWS (nd_config.flags & ndGF_CAPTURE_UNKNOWN_FLOWS)
-#define ND_PRIVATE_EXTADDR (nd_config.flags & ndGF_PRIVATE_EXTADDR)
-#define ND_SSL_USE_TLSv1 (nd_config.flags & ndGF_SSL_USE_TLSv1)
-#define ND_SSL_VERIFY (nd_config.flags & ndGF_SSL_VERIFY)
-#define ND_USE_CONNTRACK (nd_config.flags & ndGF_USE_CONNTRACK)
-#define ND_USE_NETLINK (nd_config.flags & ndGF_USE_NETLINK)
-#define ND_USE_SINK (nd_config.flags & ndGF_USE_SINK)
-#define ND_USE_DHC (nd_config.flags & ndGF_USE_DHC)
-#define ND_USE_FHC (nd_config.flags & ndGF_USE_FHC)
-#define ND_EXPORT_JSON (nd_config.flags & ndGF_EXPORT_JSON)
-#define ND_VERBOSE (nd_config.flags & ndGF_VERBOSE)
-#define ND_REPLAY_DELAY (nd_config.flags & ndGF_REPLAY_DELAY)
-#define ND_REMAIN_IN_FOREGROUND (nd_config.flags & ndGF_REMAIN_IN_FOREGROUND)
-#define ND_FLOW_DUMP_ESTABLISHED (nd_config.flags & ndGF_FLOW_DUMP_ESTABLISHED)
-#define ND_FLOW_DUMP_UNKNOWN (nd_config.flags & ndGF_FLOW_DUMP_UNKNOWN)
-#define ND_UPLOAD_ENABLED (nd_config.flags & ndGF_UPLOAD_ENABLED)
-#define ND_UPLOAD_NAT_FLOWS (nd_config.flags & ndGF_UPLOAD_NAT_FLOWS)
-#define ND_WAIT_FOR_CLIENT (nd_config.flags & ndGF_WAIT_FOR_CLIENT)
-
-#define ND_GF_SET_FLAG(flag, value) \
-{ \
-    if (value) nd_config.flags |= flag; \
-    else nd_config.flags &= ~flag; \
-}
-
-typedef struct nd_global_config_t {
-    char *path_config;
-    char *path_sink_config;
-    char *path_export_json;
-    char *path_uuid;
-    char *path_uuid_serial;
-    char *path_uuid_site;
-    char *url_sink;
-    char *url_sink_provision;
-    char *uuid;
-    char *uuid_serial;
-    char *uuid_site;
-    size_t max_backlog;
-    size_t max_packet_queue;
-    uint32_t flags;
-    uint8_t digest_sink_config[SHA1_DIGEST_LENGTH];
-    unsigned max_fhc;
-    unsigned max_flows;
-    unsigned max_tcp_pkts;
-    unsigned max_udp_pkts;
-    unsigned sink_max_post_errors;
-    unsigned sink_connect_timeout;
-    unsigned sink_xfer_timeout;
-    unsigned ttl_dns_entry;
-    unsigned ttl_idle_flow;
-    unsigned ttl_idle_tcp_flow;
-    unsigned update_interval;
-    unsigned update_imf;
-    int16_t ca_capture_base;
-    int16_t ca_conntrack;
-    int16_t ca_detection_base;
-    int16_t ca_detection_cores;
-    int16_t ca_sink;
-    int16_t ca_socket;
-    FILE *h_flow;
-    enum nd_dhc_save dhc_save;
-    enum nd_fhc_save fhc_save;
-    unsigned fhc_purge_divisor;
-
-    vector<pair<string, string> > socket_host;
-    vector<string> socket_path;
-    vector<struct sockaddr *> privacy_filter_host;
-    vector<uint8_t *> privacy_filter_mac;
-    vector<pair<regex *, string> > privacy_regex;
-    nd_device_filter device_filters;
-#ifdef _ND_USE_PLUGINS
-    map<string, string> plugin_services;
-    map<string, string> plugin_tasks;
-    map<string, string> plugin_detections;
-    map<string, string> plugin_stats;
-#endif
-    map<string, string> custom_headers;
-} nd_global_config;
-
 typedef struct nd_agent_stats_t
 {
     long cpus;
@@ -355,6 +245,7 @@ typedef struct nd_packet_stats_t
         uint64_t igmp;
         uint64_t tcp;
         uint64_t tcp_seq_error;
+        uint64_t tcp_resets;
         uint64_t udp;
         uint64_t ip_bytes;
         uint64_t ip4_bytes;
@@ -387,6 +278,7 @@ typedef struct nd_packet_stats_t
         pkt.igmp += rhs.pkt.igmp;
         pkt.tcp += rhs.pkt.tcp;
         pkt.tcp_seq_error += rhs.pkt.tcp_seq_error;
+        pkt.tcp_resets += rhs.pkt.tcp_resets;
         pkt.udp += rhs.pkt.udp;
         pkt.ip_bytes += rhs.pkt.ip_bytes;
         pkt.ip4_bytes += rhs.pkt.ip4_bytes;
