@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,13 +22,13 @@
 #include <set>
 #include <stdexcept>
 #include <unordered_map>
+#include <unordered_set>
 #include <list>
 #include <vector>
 #include <sstream>
-#ifdef HAVE_ATOMIC
 #include <atomic>
-#endif
 #include <regex>
+#include <mutex>
 
 #include <sys/stat.h>
 #include <sys/select.h>
@@ -68,6 +68,10 @@ using namespace std;
 #include "nd-netlink.h"
 #endif
 #include "nd-json.h"
+#include "nd-apps.h"
+#include "nd-protos.h"
+#include "nd-risks.h"
+#include "nd-category.h"
 #include "nd-flow.h"
 #include "nd-thread.h"
 #include "nd-util.h"
@@ -77,8 +81,6 @@ using namespace std;
 
 #include "nd-conntrack.h"
 
-extern nd_global_config nd_config;
-
 static time_t nd_ct_last_flow_purge_ttl = 0;
 
 static int nd_ct_event_callback(
@@ -528,7 +530,7 @@ void ndConntrackThread::PrintFlow(ndFlow *flow, string &text)
     text = os.str();
 }
 
-void ndConntrackThread::ClassifyFlow(ndFlow *flow)
+void ndConntrackThread::UpdateFlow(ndFlow *flow)
 {
     sha1 ctx;
     string digest;
@@ -581,6 +583,13 @@ void ndConntrackThread::ClassifyFlow(ndFlow *flow)
 
         ndConntrackFlow *ct_flow = flow_iter->second;
 
+        ct_flow->updated_at = time(NULL);
+
+#if defined(_ND_WITH_CONNTRACK_MDATA)
+        flow->ct_id = ct_flow->id;
+        flow->ct_mark = ct_flow->mark;
+#endif
+
         switch (ct_flow->l3_proto) {
         case AF_INET:
             sa_orig_src = reinterpret_cast<struct sockaddr_in *>(
@@ -599,14 +608,8 @@ void ndConntrackThread::ClassifyFlow(ndFlow *flow)
             }
 #endif
             if (memcmp(sa_orig_src, sa_repl_dst, sizeof(struct sockaddr_in)) ||
-                memcmp(sa_orig_dst, sa_repl_src, sizeof(struct sockaddr_in))) {
+                memcmp(sa_orig_dst, sa_repl_src, sizeof(struct sockaddr_in)))
                 flow->flags.ip_nat = true;
-#ifdef _ND_USE_CONNTRACK
-                flow->ct_id = ct_flow->id;
-                flow->ct_mark = ct_flow->mark;
-#endif
-                ct_flow->updated_at = time(NULL);
-            }
 
             break;
 
@@ -627,14 +630,8 @@ void ndConntrackThread::ClassifyFlow(ndFlow *flow)
             }
 #endif
             if (memcmp(sa6_orig_src, sa6_repl_dst, sizeof(struct sockaddr_in6)) ||
-                memcmp(sa6_orig_dst, sa6_repl_src, sizeof(struct sockaddr_in6))) {
+                memcmp(sa6_orig_dst, sa6_repl_src, sizeof(struct sockaddr_in6)))
                 flow->flags.ip_nat = true;
-#ifdef _ND_USE_CONNTRACK
-                flow->ct_id = ct_flow->id;
-                flow->ct_mark = ct_flow->mark;
-#endif
-                ct_flow->updated_at = time(NULL);
-            }
 
             break;
         }
