@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,10 +23,13 @@
 #include <map>
 #include <list>
 #include <vector>
+#include <set>
 #include <atomic>
 #include <unordered_map>
+#include <unordered_set>
 #include <sstream>
 #include <regex>
+#include <mutex>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -49,22 +52,28 @@ using namespace std;
 
 #include "netifyd.h"
 
+#include "nd-config.h"
 #include "nd-ndpi.h"
 #ifdef _ND_USE_NETLINK
 #include "nd-netlink.h"
 #endif
 #include "nd-json.h"
 #include "nd-util.h"
+#include "nd-apps.h"
+#include "nd-category.h"
+#include "nd-protos.h"
+#include "nd-risks.h"
 #include "nd-flow.h"
 
 // Enable flow hash cache debug logging
 //#define _ND_DEBUG_FHC 1
 
-extern nd_global_config nd_config;
-extern nd_device_ethers device_ethers;
+extern nd_global_config *nd_config;
 
-ndFlow::ndFlow(nd_ifaces::iterator iface)
-    : iface(iface), dpi_thread_id(-1), pkt(NULL),
+nd_device_ether nd_device_ethers;
+
+ndFlow::ndFlow(nd_interface::iterator iface)
+    : iface(iface), dpi_thread_id(-1),
     ip_version(0), ip_protocol(0), vlan_id(0), tcp_last_seq(0),
     ts_first_seen(0), ts_first_update(0), ts_last_seen(0),
     lower_map(LOWER_UNKNOWN), other_type(OTHER_UNKNOWN),
@@ -72,23 +81,28 @@ ndFlow::ndFlow(nd_ifaces::iterator iface)
     lower_addr4(NULL), lower_addr6(NULL), upper_addr4(NULL), upper_addr6(NULL),
     lower_ip{}, upper_ip{},
     lower_port(0), upper_port(0),
-    tunnel_type(TUNNEL_NONE), gtp{},
+    tunnel_type(TUNNEL_NONE),
     lower_bytes(0), upper_bytes(0), total_bytes(0),
     lower_packets(0), upper_packets(0), total_packets(0),
-    detection_packets(0), detected_protocol{},
-    detected_protocol_name(NULL), detected_application_name(NULL),
-    ndpi_flow(NULL), id_src(NULL), id_dst(NULL),
+    detection_packets(0),
+    detected_protocol(ND_PROTO_UNKNOWN),
+    detected_application(ND_APP_UNKNOWN),
+    detected_protocol_name("Unknown"),
+    detected_application_name(NULL),
+    category { ND_CAT_UNKNOWN, ND_CAT_UNKNOWN, ND_CAT_UNKNOWN },
+    ndpi_flow(NULL),
     digest_lower{}, digest_mdata{},
-    host_server_name{}, http{},
+    dns_host_name{}, host_server_name{}, http{},
     privacy_mask(0), origin(0), direction(0),
     capture_filename{},
-#ifdef _ND_USE_CONNTRACK
+#if defined(_ND_USE_CONNTRACK) && defined(_ND_WITH_CONNTRACK_MDATA)
     ct_id(0), ct_mark(0),
 #endif
 #ifdef _ND_USE_NETLINK
     lower_type(ndNETLINK_ATYPE_UNKNOWN), upper_type(ndNETLINK_ATYPE_UNKNOWN),
 #endif
-    flags{}
+    flags{}, queued(0), gtp{},
+    risks{}, ndpi_risk_score(0), ndpi_risk_score_client(0), ndpi_risk_score_server(0)
 {
     lower_addr4 = (struct sockaddr_in *)&lower_addr;
     lower_addr6 = (struct sockaddr_in6 *)&lower_addr;
@@ -99,7 +113,7 @@ ndFlow::ndFlow(nd_ifaces::iterator iface)
 }
 
 ndFlow::ndFlow(const ndFlow &flow)
-    : iface(flow.iface), dpi_thread_id(-1), pkt(NULL),
+    : iface(flow.iface), dpi_thread_id(-1),
     ip_version(flow.ip_version), ip_protocol(flow.ip_protocol),
     vlan_id(flow.vlan_id), tcp_last_seq(flow.tcp_last_seq),
     ts_first_seen(flow.ts_first_seen), ts_first_update(flow.ts_first_update),
@@ -108,22 +122,27 @@ ndFlow::ndFlow(const ndFlow &flow)
     lower_addr(flow.lower_addr), upper_addr(flow.upper_addr),
     lower_ip{}, upper_ip{},
     lower_port(flow.lower_port), upper_port(flow.upper_port),
-    tunnel_type(flow.tunnel_type), gtp(flow.gtp),
+    tunnel_type(flow.tunnel_type),
     lower_bytes(0), upper_bytes(0), total_bytes(0),
     lower_packets(0), upper_packets(0), total_packets(0),
-    detection_packets(0), detected_protocol{},
-    detected_protocol_name(NULL), detected_application_name(NULL),
-    ndpi_flow(NULL), id_src(NULL), id_dst(NULL),
-    host_server_name{}, http{},
+    detection_packets(0),
+    detected_protocol(ND_PROTO_UNKNOWN),
+    detected_application(ND_APP_UNKNOWN),
+    detected_protocol_name("Unknown"),
+    detected_application_name(NULL),
+    category { ND_CAT_UNKNOWN, ND_CAT_UNKNOWN, ND_CAT_UNKNOWN },
+    ndpi_flow(NULL),
+    dns_host_name{}, host_server_name{}, http{},
     privacy_mask(0), origin(0), direction(0),
     capture_filename{},
-#ifdef _ND_USE_CONNTRACK
+#if defined(_ND_USE_CONNTRACK) && defined(_ND_WITH_CONNTRACK_MDATA)
     ct_id(0), ct_mark(0),
 #endif
 #ifdef _ND_USE_NETLINK
     lower_type(ndNETLINK_ATYPE_UNKNOWN), upper_type(ndNETLINK_ATYPE_UNKNOWN),
 #endif
-    flags{}
+    flags{}, queued(0), gtp(flow.gtp),
+    risks{}, ndpi_risk_score(0), ndpi_risk_score_client(0), ndpi_risk_score_server(0)
 {
     memcpy(lower_mac, flow.lower_mac, ETH_ALEN);
     memcpy(upper_mac, flow.upper_mac, ETH_ALEN);
@@ -141,14 +160,20 @@ ndFlow::~ndFlow()
 {
     release();
 
-    if (detected_protocol_name != NULL) {
-        free(detected_protocol_name);
-        detected_protocol_name = NULL;
-    }
     if (detected_application_name != NULL) {
         free(detected_application_name);
         detected_application_name = NULL;
     }
+
+    if (has_ssl_issuer_dn()) {
+        free(ssl.issuer_dn);
+        ssl.issuer_dn = NULL;
+    }
+
+    if (has_ssl_subject_dn()) {
+        free(ssl.subject_dn);
+        ssl.subject_dn = NULL;
+    }
 }
 
 void ndFlow::hash(const string &device,
@@ -205,22 +230,15 @@ void ndFlow::hash(const string &device,
 
         if (host_server_name[0] != '\0') {
             sha1_write(&ctx,
-                host_server_name, strnlen(host_server_name, ND_MAX_HOSTNAME));
+                host_server_name, strnlen(host_server_name, ND_FLOW_HOSTNAME));
         }
         if (has_ssl_client_sni()) {
             sha1_write(&ctx,
-                ssl.client_sni, strnlen(ssl.client_sni, ND_FLOW_TLS_CNLEN));
-        }
-        if (has_ssl_server_cn()) {
-            sha1_write(&ctx,
-                ssl.server_cn, strnlen(ssl.server_cn, ND_FLOW_TLS_CNLEN));
+                ssl.client_sni, strnlen(ssl.client_sni, ND_FLOW_HOSTNAME));
         }
         if (has_bt_info_hash()) {
             sha1_write(&ctx, bt.info_hash, ND_FLOW_BTIHASH_LEN);
         }
-        if (has_mdns_answer()) {
-            sha1_write(&ctx, mdns.answer, ND_FLOW_MDNS_ANSLEN);
-        }
     }
 
     if (key != NULL && key_length > 0)
@@ -283,10 +301,7 @@ void ndFlow::reset(void)
 
 void ndFlow::release(void)
 {
-    if (pkt != NULL) { delete [] pkt; pkt = NULL; }
     if (ndpi_flow != NULL) { ndpi_free_flow(ndpi_flow); ndpi_flow = NULL; }
-    if (id_src != NULL) { delete id_src; id_src = NULL; }
-    if (id_dst != NULL) { delete id_dst; id_dst = NULL; }
 
     for (nd_flow_capture::const_iterator i = capture.begin();
         i != capture.end(); i++) {
@@ -297,49 +312,42 @@ void ndFlow::release(void)
     capture.clear();
 }
 
-uint16_t ndFlow::master_protocol(void) const
+nd_proto_id_t ndFlow::master_protocol(void) const
 {
-    uint16_t proto = (detected_protocol.master_protocol !=
-        NDPI_PROTOCOL_UNKNOWN) ?
-            detected_protocol.master_protocol :
-            detected_protocol.app_protocol;
-
-    switch (proto) {
-    case NDPI_PROTOCOL_GMAIL:
-    case NDPI_PROTOCOL_HTTPS:
-    case NDPI_PROTOCOL_MAIL_IMAP:
-    case NDPI_PROTOCOL_MAIL_IMAPS:
-    case NDPI_PROTOCOL_MAIL_POPS:
-    case NDPI_PROTOCOL_MAIL_SMTPS:
-    case NDPI_PROTOCOL_OSCAR:
-    case NDPI_PROTOCOL_SSL:
-    case NDPI_PROTOCOL_SSL_NO_CERT:
-    case NDPI_PROTOCOL_TOR:
-    case NDPI_PROTOCOL_UNENCRYPTED_JABBER:
-        return NDPI_PROTOCOL_SSL;
-    case NDPI_PROTOCOL_FACEBOOK:
-    case NDPI_PROTOCOL_HTTP:
-    case NDPI_PROTOCOL_HTTP_CONNECT:
-    case NDPI_PROTOCOL_HTTP_PROXY:
-    case NDPI_PROTOCOL_NETFLIX:
-    case NDPI_PROTOCOL_OOKLA:
-    case NDPI_PROTOCOL_PPSTREAM:
-    case NDPI_PROTOCOL_QQ:
-    case NDPI_PROTOCOL_RTSP:
-    case NDPI_PROTOCOL_STEAM:
-    case NDPI_PROTOCOL_TEAMVIEWER:
-    case NDPI_PROTOCOL_XBOX:
-        return NDPI_PROTOCOL_HTTP;
+    switch (detected_protocol) {
+    case ND_PROTO_HTTPS:
+    case ND_PROTO_TLS:
+    case ND_PROTO_FTPS_CONTROL:
+    case ND_PROTO_FTPS_DATA:
+    case ND_PROTO_MAIL_IMAPS:
+    case ND_PROTO_MAIL_POPS:
+    case ND_PROTO_MAIL_SMTPS:
+    case ND_PROTO_MQTTS:
+    case ND_PROTO_NNTPS:
+    case ND_PROTO_SIPS:
+        return ND_PROTO_TLS;
+    case ND_PROTO_HTTP:
+    case ND_PROTO_HTTP_CONNECT:
+    case ND_PROTO_HTTP_PROXY:
+    case ND_PROTO_OOKLA:
+    case ND_PROTO_PPSTREAM:
+    case ND_PROTO_QQ:
+    case ND_PROTO_RTSP:
+    case ND_PROTO_STEAM:
+    case ND_PROTO_TEAMVIEWER:
+    case ND_PROTO_XBOX:
+        return ND_PROTO_HTTP;
+    default:
+        break;
     }
 
-    return proto;
+    return detected_protocol;
 }
 
 bool ndFlow::has_dhcp_fingerprint(void) const
 {
     return (
-        (detected_protocol.master_protocol == NDPI_PROTOCOL_DHCP ||
-        detected_protocol.app_protocol == NDPI_PROTOCOL_DHCP) &&
+        detected_protocol == ND_PROTO_DHCP &&
         dhcp.fingerprint[0] != '\0'
     );
 }
@@ -347,8 +355,7 @@ bool ndFlow::has_dhcp_fingerprint(void) const
 bool ndFlow::has_dhcp_class_ident(void) const
 {
     return (
-        (detected_protocol.master_protocol == NDPI_PROTOCOL_DHCP ||
-        detected_protocol.app_protocol == NDPI_PROTOCOL_DHCP) &&
+        detected_protocol == ND_PROTO_DHCP &&
         dhcp.class_ident[0] != '\0'
     );
 }
@@ -356,7 +363,7 @@ bool ndFlow::has_dhcp_class_ident(void) const
 bool ndFlow::has_http_user_agent(void) const
 {
     return (
-        master_protocol() == NDPI_PROTOCOL_HTTP &&
+        master_protocol() == ND_PROTO_HTTP &&
         http.user_agent[0] != '\0'
     );
 }
@@ -371,8 +378,7 @@ bool ndFlow::has_http_url(void) const
 bool ndFlow::has_ssh_client_agent(void) const
 {
     return (
-        (detected_protocol.master_protocol == NDPI_PROTOCOL_SSH ||
-        detected_protocol.app_protocol == NDPI_PROTOCOL_SSH) &&
+        detected_protocol == ND_PROTO_SSH &&
         ssh.client_agent[0] != '\0'
     );
 }
@@ -380,8 +386,7 @@ bool ndFlow::has_ssh_client_agent(void) const
 bool ndFlow::has_ssh_server_agent(void) const
 {
     return (
-        (detected_protocol.master_protocol == NDPI_PROTOCOL_SSH ||
-        detected_protocol.app_protocol == NDPI_PROTOCOL_SSH) &&
+        detected_protocol == ND_PROTO_SSH &&
         ssh.server_agent[0] != '\0'
     );
 }
@@ -389,31 +394,39 @@ bool ndFlow::has_ssh_server_agent(void) const
 bool ndFlow::has_ssl_client_sni(void) const
 {
     return (
-        master_protocol() == NDPI_PROTOCOL_SSL &&
-        ssl.client_sni[0] != '\0'
+        (master_protocol() == ND_PROTO_TLS || detected_protocol == ND_PROTO_QUIC) &&
+        ssl.client_sni != NULL && ssl.client_sni[0] != '\0'
     );
 }
 
 bool ndFlow::has_ssl_server_cn(void) const
 {
     return (
-        master_protocol() == NDPI_PROTOCOL_SSL &&
+        (master_protocol() == ND_PROTO_TLS || detected_protocol == ND_PROTO_QUIC) &&
         ssl.server_cn[0] != '\0'
     );
 }
 
-bool ndFlow::has_ssl_server_organization(void) const
+bool ndFlow::has_ssl_issuer_dn(void) const
 {
     return (
-        master_protocol() == NDPI_PROTOCOL_SSL &&
-        ssl.server_organization[0] != '\0'
+        (master_protocol() == ND_PROTO_TLS || detected_protocol == ND_PROTO_QUIC) &&
+        ssl.issuer_dn != NULL
+    );
+}
+
+bool ndFlow::has_ssl_subject_dn(void) const
+{
+    return (
+        (master_protocol() == ND_PROTO_TLS || detected_protocol == ND_PROTO_QUIC) &&
+        ssl.subject_dn != NULL
     );
 }
 
 bool ndFlow::has_ssl_client_ja3(void) const
 {
     return (
-        master_protocol() == NDPI_PROTOCOL_SSL &&
+        master_protocol() == ND_PROTO_TLS &&
         ssl.client_ja3[0] != '\0'
     );
 }
@@ -421,7 +434,7 @@ bool ndFlow::has_ssl_client_ja3(void) const
 bool ndFlow::has_ssl_server_ja3(void) const
 {
     return (
-        master_protocol() == NDPI_PROTOCOL_SSL &&
+        master_protocol() == ND_PROTO_TLS &&
         ssl.server_ja3[0] != '\0'
     );
 }
@@ -429,47 +442,45 @@ bool ndFlow::has_ssl_server_ja3(void) const
 bool ndFlow::has_bt_info_hash(void) const
 {
     return (
-        (detected_protocol.master_protocol == NDPI_PROTOCOL_BITTORRENT ||
-        detected_protocol.app_protocol == NDPI_PROTOCOL_BITTORRENT) &&
+        detected_protocol == ND_PROTO_BITTORRENT &&
         bt.info_hash_valid
     );
 }
 
-bool ndFlow::has_mdns_answer(void) const
-{
-    return (
-        (detected_protocol.master_protocol == NDPI_PROTOCOL_MDNS ||
-        detected_protocol.app_protocol == NDPI_PROTOCOL_MDNS) &&
-        mdns.answer[0] != '\0'
-    );
-}
-
 bool ndFlow::has_ssdp_headers(void) const
 {
     return (
-        detected_protocol.master_protocol == NDPI_PROTOCOL_SSDP &&
+        detected_protocol == ND_PROTO_SSDP &&
         ssdp.headers.size()
     );
 }
-
+#if 0
+bool ndFlow::has_mining_variant(void) const
+{
+    return (
+        detected_protocol == ND_PROTO_MINING &&
+        mining.variant[0] != '\0'
+    );
+}
+#endif
 void ndFlow::print(void) const
 {
     const char *lower_name = lower_ip, *upper_name = upper_ip;
 
     if (ND_DEBUG_WITH_ETHERS) {
         string key;
-        nd_device_ethers::const_iterator i;
+        nd_device_ether::const_iterator i;
 
         key.assign((const char *)lower_mac, ETH_ALEN);
 
-        i = device_ethers.find(key);
-        if (i != device_ethers.end())
+        i = nd_device_ethers.find(key);
+        if (i != nd_device_ethers.end())
             lower_name = i->second.c_str();
 
         key.assign((const char *)upper_mac, ETH_ALEN);
 
-        i = device_ethers.find(key);
-        if (i != device_ethers.end())
+        i = nd_device_ethers.find(key);
+        if (i != nd_device_ethers.end())
             upper_name = i->second.c_str();
     }
 
@@ -480,17 +491,19 @@ void ndFlow::print(void) const
     nd_sha1_to_string((const uint8_t *)bt.info_hash, digest);
 
     nd_flow_printf(
-        "%s: [%c%c%c%c%c%c] %s%s%s %s:%hu %c%c%c %s:%hu%s%s%s%s%s%s%s%s%s\n",
+        "%s: [%c%c%c%c%c%c%c%c] %s%s%s %s:%hu %c%c%c %s:%hu%s%s%s%s%s%s%s\n",
         iface_name.c_str(),
         (iface->first) ? 'i' : 'e',
         (ip_version == 4) ? '4' : (ip_version == 6) ? '6' : '-',
         flags.ip_nat.load() ? 'n' : '-',
+        (flags.detection_updated.load()) ? 'u' : '-',
         (flags.detection_guessed.load()) ? 'g' : '-',
         (flags.dhc_hit.load()) ? 'd' : '-',
         (privacy_mask & PRIVATE_LOWER) ? 'p' :
             (privacy_mask & PRIVATE_UPPER) ? 'P' :
             (privacy_mask & (PRIVATE_LOWER | PRIVATE_UPPER)) ? 'X' :
             '-',
+        (flags.soft_dissector.load()) ? 's' : '-',
         detected_protocol_name,
         (detected_application_name != NULL) ? "." : "",
         (detected_application_name != NULL) ? detected_application_name : "",
@@ -499,23 +512,23 @@ void ndFlow::print(void) const
         (origin == ORIGIN_UNKNOWN) ? '?' : '-',
         (origin == ORIGIN_UPPER || origin == ORIGIN_UNKNOWN) ? '-' : '>',
         upper_name, ntohs(upper_port),
-        (host_server_name[0] != '\0' || has_mdns_answer()) ? " H: " : "",
-        (host_server_name[0] != '\0' || has_mdns_answer()) ?
-            has_mdns_answer() ? mdns.answer : host_server_name : "",
-        (has_ssl_client_sni() || has_ssl_server_cn()) ? " SSL" : "",
+        (dns_host_name[0] != '\0' || host_server_name[0] != '\0') ? " H: " : "",
+        (host_server_name[0] != '\0') ? host_server_name : (
+            (dns_host_name[0] != '\0') ? dns_host_name : ""
+        ),
+        (has_ssl_client_sni()) ? " SSL" : "",
         (has_ssl_client_sni()) ? " C: " : "",
         (has_ssl_client_sni()) ? ssl.client_sni : "",
-        (has_ssl_server_cn()) ? " S: " : "",
-        (has_ssl_server_cn()) ? ssl.server_cn : "",
         (has_bt_info_hash()) ? " BT-IH: " : "",
         (has_bt_info_hash()) ? digest.c_str() : ""
     );
-
+#if 0
     if (ND_DEBUG &&
-        detected_protocol.master_protocol == NDPI_PROTOCOL_SSL &&
+        detected_protocol == ND_PROTO_TLS &&
         flags.detection_guessed.load() == false && ssl.version == 0x0000) {
         nd_dprintf("%s: SSL with no SSL/TLS verison.\n", iface->second.c_str());
     }
+#endif
 }
 
 void ndFlow::update_lower_maps(void)
@@ -696,7 +709,8 @@ void ndFlow::json_encode(json &j, uint8_t encode_includes)
     if (encode_includes & ENCODE_METADATA) {
         j["ip_nat"] = (bool)flags.ip_nat.load();
         j["dhc_hit"] = (bool)flags.dhc_hit.load();
-#ifdef _ND_USE_CONNTRACK
+        j["soft_dissector"] = (bool)flags.soft_dissector.load();
+#if defined(_ND_USE_CONNTRACK) && defined(_ND_WITH_CONNTRACK_MDATA)
         j["ct_id"] = ct_id;
         j["ct_mark"] = ct_mark;
 #endif
@@ -770,17 +784,23 @@ void ndFlow::json_encode(json &j, uint8_t encode_includes)
         j[_lower_port] = (unsigned)ntohs(lower_port);
         j[_upper_port] = (unsigned)ntohs(upper_port);
 
-        j["detected_protocol"] =
-            (unsigned)detected_protocol.master_protocol;
+        j["detected_protocol"] = (unsigned)detected_protocol;
         j["detected_protocol_name"] =
             (detected_protocol_name != NULL) ? detected_protocol_name : "Unknown";
 
-        j["detected_application"] =
-            (unsigned)detected_protocol.app_protocol;
+        j["detected_application"] = (unsigned)detected_application;
         j["detected_application_name"] =
             (detected_application_name != NULL) ? detected_application_name : "Unknown";
 
-        j["detection_guessed"] = (unsigned)flags.detection_guessed.load();
+        j["detection_guessed"] = flags.detection_guessed.load();
+        j["detection_updated"] = flags.detection_updated.load();
+
+        j["category"]["application"] = category.application;
+        j["category"]["protocol"] = category.protocol;
+        j["category"]["domain"] = category.domain;
+
+        if (dns_host_name[0] != '\0')
+            j["dns_host_name"] = dns_host_name;
 
         if (host_server_name[0] != '\0')
             j["host_server_name"] = host_server_name;
@@ -811,7 +831,8 @@ void ndFlow::json_encode(json &j, uint8_t encode_includes)
                 j["ssh"]["server"] = ssh.server_agent;
         }
 
-        if (has_ssl_client_sni() || has_ssl_server_cn()) {
+        if (master_protocol() == ND_PROTO_TLS
+            || detected_protocol == ND_PROTO_QUIC) {
 
             char tohex[7];
 
@@ -827,8 +848,11 @@ void ndFlow::json_encode(json &j, uint8_t encode_includes)
             if (has_ssl_server_cn())
                 j["ssl"]["server_cn"] = ssl.server_cn;
 
-            if (has_ssl_server_organization())
-                j["ssl"]["organization"] = ssl.server_organization;
+            if (has_ssl_issuer_dn())
+                j["ssl"]["issuer_dn"] = ssl.issuer_dn;
+
+            if (has_ssl_subject_dn())
+                j["ssl"]["subject_dn"] = ssl.subject_dn;
 
             if (has_ssl_client_ja3())
                 j["ssl"]["client_ja3"] = ssl.client_ja3;
@@ -840,27 +864,29 @@ void ndFlow::json_encode(json &j, uint8_t encode_includes)
                 nd_sha1_to_string((const uint8_t *)ssl.cert_fingerprint, digest);
                 j["ssl"]["fingerprint"] = digest;
             }
+
+            j["ssl"]["alpn"] = ssl.alpn;
+            j["ssl"]["alpn_server"] = ssl.alpn_server;
         }
 
         if (has_bt_info_hash()) {
-
             nd_sha1_to_string((const uint8_t *)bt.info_hash, digest);
             j["bt"]["info_hash"] = digest;
         }
 
-        if (has_mdns_answer()) {
-
-            j["mdns"]["answer"] = mdns.answer;
-        }
-
-        if (has_ssdp_headers()) {
-
+        if (has_ssdp_headers())
             j["ssdp"] = ssdp.headers;
-
-        }
-
+#if 0
+        if (has_mining_variant())
+            j["mining"]["variant"] = mining.variant;
+#endif
         j["first_seen_at"] = ts_first_seen;
         j["first_update_at"] = ts_first_update;
+
+        j["risks"]["risks"] = risks;
+        j["risks"]["ndpi_risk_score"] = ndpi_risk_score;
+        j["risks"]["ndpi_risk_score_client"] = ndpi_risk_score_client;
+        j["risks"]["ndpi_risk_score_server"] = ndpi_risk_score_server;
     }
 
     if (encode_includes & ENCODE_TUNNELS) {
