@@ -1,5 +1,5 @@
-// Netify Agent
-// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
+// Netify Agent Plugin Example
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,12 +20,17 @@
 
 #include <stdexcept>
 #include <vector>
+#include <set>
 #include <map>
+#include <queue>
 #include <unordered_map>
+#include <unordered_set>
 #include <string>
+#include <fstream>
 #include <sstream>
 #include <atomic>
 #include <regex>
+#include <mutex>
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -58,8 +63,16 @@ class ndPluginLoader;
 #include <nd-util.h>
 #include <nd-thread.h>
 #include <nd-netlink.h>
+#include <nd-apps.h>
+#include <nd-protos.h>
+#include <nd-category.h>
 #include <nd-flow.h>
+class ndFlowMap;
 #include <nd-plugin.h>
+#if (_ND_PLUGIN_VER > 0x20211111)
+#include <nd-flow-map.h>
+#endif
+#include <nd-category.h>
 
 #include "plugin-example.h"
 
