@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,11 +21,12 @@
 #include <stdexcept>
 #include <vector>
 #include <map>
+#include <set>
 #include <unordered_map>
-#ifdef HAVE_ATOMIC
+#include <unordered_set>
 #include <atomic>
-#endif
 #include <regex>
+#include <mutex>
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -58,7 +59,12 @@ using namespace std;
 #include "nd-thread.h"
 #include "nd-json.h"
 #include "nd-util.h"
+#include "nd-apps.h"
+#include "nd-protos.h"
+#include "nd-risks.h"
+#include "nd-category.h"
 #include "nd-flow.h"
+#include "nd-flow-map.h"
 #include "nd-plugin.h"
 
 ndPlugin::ndPlugin(const string &tag)
@@ -208,6 +214,18 @@ ndPluginDetection::~ndPluginDetection()
     nd_dprintf("Plugin detection destroyed: %s\n", tag.c_str());
 }
 
+ndPluginStats::ndPluginStats(const string &tag)
+    : ndPlugin(tag)
+{
+    type = ndPlugin::TYPE_STATS;
+    nd_dprintf("Plugin detection initialized: %s\n", tag.c_str());
+}
+
+ndPluginStats::~ndPluginStats()
+{
+    nd_dprintf("Plugin detection destroyed: %s\n", tag.c_str());
+}
+
 ndPluginLoader::ndPluginLoader(const string &so_name, const string &tag)
     : so_name(so_name), so_handle(NULL)
 {
