@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -65,6 +65,7 @@
 #include <pwd.h>
 #include <grp.h>
 #include <libgen.h>
+#include <dirent.h>
 
 #include <arpa/inet.h>
 #include <netdb.h>
@@ -72,10 +73,6 @@
 
 #include <net/if.h>
 
-#ifndef AF_LINK
-#define AF_LINK AF_PACKET
-#endif
-
 #include <pcap/pcap.h>
 
 #include <nlohmann/json.hpp>
@@ -85,12 +82,15 @@ using namespace std;
 
 #include "netifyd.h"
 
+#include "nd-config.h"
 #include "nd-ndpi.h"
 #include "nd-sha1.h"
 #include "nd-json.h"
 #include "nd-util.h"
 
-extern nd_global_config nd_config;
+extern nd_global_config *nd_config;
+
+pthread_mutex_t *nd_printf_mutex = NULL;
 
 ndException::ndException(const string &where_arg, const string &what_arg) throw()
     : runtime_error(what_arg), where_arg(where_arg), what_arg(what_arg), message(NULL)
@@ -140,8 +140,6 @@ void nd_mem_free(void *ptr)
     free(ptr);
 }
 
-extern pthread_mutex_t *nd_printf_mutex;
-
 void nd_printf(const char *format, ...)
 {
     if (ND_QUIET) return;
@@ -190,7 +188,7 @@ void nd_flow_printf(const char *format, ...)
 
     va_list ap;
     va_start(ap, format);
-    vfprintf(nd_config.h_flow, format, ap);
+    vfprintf(nd_config->h_flow, format, ap);
     va_end(ap);
 
     pthread_mutex_unlock(nd_printf_mutex);
@@ -201,14 +199,14 @@ void nd_ndpi_debug_printf(uint32_t protocol, void *ndpi,
     ndpi_log_level_t level, const char *file, const char *func, unsigned line,
     const char *format, ...)
 {
-    if (ND_DEBUG) {
+    if (ND_DEBUG && (ND_DEBUG_NDPI || level == NDPI_LOG_ERROR)) {
 
         pthread_mutex_lock(nd_printf_mutex);
 
         va_list ap;
         va_start(ap, format);
 
-        fprintf(stderr, "[nDPI:%08x:%p:%s]: %s/%s:%d: ", protocol, ndpi,
+        fprintf(stdout, "[nDPI:%08x:%p:%s]: %s/%s:%d: ", protocol, ndpi,
             (level == NDPI_LOG_ERROR) ? "ERROR" :
                 (level == NDPI_LOG_TRACE) ? "TRACE" :
                     (level == NDPI_LOG_DEBUG) ? "DEBUG" :
@@ -216,8 +214,8 @@ void nd_ndpi_debug_printf(uint32_t protocol, void *ndpi,
                             "UNK???",
             file, func, line
         );
-        vfprintf(stderr, format, ap);
 
+        vfprintf(stdout, format, ap);
         va_end(ap);
 
         pthread_mutex_unlock(nd_printf_mutex);
@@ -314,20 +312,28 @@ void nd_print_number(ostringstream &os, uint64_t value, bool units_binary)
     }
 }
 
-static inline void nd_ltrim(string &s)
+void nd_ltrim(string &s, unsigned char c)
 {
-    s.erase(s.begin(), find_if(s.begin(), s.end(), [](unsigned char ch) {
-        return !isspace(ch);
+    s.erase(s.begin(), find_if(s.begin(), s.end(), [c](unsigned char ch) {
+        if (c == 0) return !isspace(ch);
+        else return (ch != c);
     }));
 }
 
-static inline void nd_rtrim(string &s)
+void nd_rtrim(string &s, unsigned char c)
 {
-    s.erase(find_if(s.rbegin(), s.rend(), [](unsigned char ch) {
-        return !isspace(ch);
+    s.erase(find_if(s.rbegin(), s.rend(), [c](unsigned char ch) {
+        if (c == 0) return !isspace(ch);
+        else return (ch != c);
     }).base(), s.end());
 }
 
+void nd_trim(string &s, unsigned char c)
+{
+    nd_ltrim(s, c);
+    nd_rtrim(s, c);
+}
+
 int nd_sha1_file(const string &filename, uint8_t *digest)
 {
     sha1 ctx;
@@ -402,28 +408,45 @@ sa_family_t nd_string_to_ip(const string &src, sockaddr_storage *ip)
     return family;
 }
 
+bool nd_ip_to_string(sa_family_t af, const void *addr, string &dst)
+{
+    char ip[INET6_ADDRSTRLEN];
+
+    switch (af) {
+    case AF_INET:
+        inet_ntop(AF_INET, addr, ip, INET_ADDRSTRLEN);
+        break;
+    case AF_INET6:
+        inet_ntop(AF_INET6, addr, ip, INET6_ADDRSTRLEN);
+        break;
+    default:
+        return false;
+    }
+
+    dst.assign(ip);
+
+    return true;
+}
+
 bool nd_ip_to_string(const sockaddr_storage &ip, string &dst)
 {
-    char addr[INET6_ADDRSTRLEN];
     const struct sockaddr_in *ipv4 = reinterpret_cast<const struct sockaddr_in *>(&ip);
     const struct sockaddr_in6 *ipv6 = reinterpret_cast<const struct sockaddr_in6 *>(&ip);
 
     switch (ip.ss_family) {
     case AF_INET:
-        inet_ntop(AF_INET, &ipv4->sin_addr.s_addr, addr, INET_ADDRSTRLEN);
-        break;
-
+        return nd_ip_to_string(
+            AF_INET, (const void *)&ipv4->sin_addr.s_addr, dst
+        );
     case AF_INET6:
-        inet_ntop(AF_INET6, &ipv6->sin6_addr.s6_addr, addr, INET6_ADDRSTRLEN);
-        break;
-
+        return nd_ip_to_string(
+            AF_INET6, (const void *)&ipv6->sin6_addr.s6_addr, dst
+        );
     default:
         return false;
     }
 
-    dst.assign(addr);
-
-    return true;
+    return false;
 }
 
 void nd_iface_name(const string &iface, string &result)
@@ -689,13 +712,27 @@ string nd_get_version_and_features(void)
 #ifdef HAVE_WORKING_REGEX
     ident << "; regex";
 #endif
-    ident << ")" <<
-        " nDPI/" << ndpi_revision() <<
-        " JSON/" << fixed << showpoint << setprecision(2) << ND_JSON_VERSION;
+    ident << ")";
 
     return ident.str();
 }
 
+bool nd_parse_app_tag(const string &tag, unsigned &id, string &name)
+{
+    id = 0;
+    name.clear();
+
+    size_t p;
+    if ((p = tag.find_first_of(".")) != string::npos) {
+        id = (unsigned)strtoul(tag.substr(0, p).c_str(), NULL, 0);
+        name = tag.substr(p + 1);
+
+        return true;
+    }
+
+    return false;
+}
+
 #ifdef _ND_USE_WATCHDOGS
 int nd_touch(const string &filename)
 {
@@ -835,7 +872,7 @@ int nd_ifreq(const string &name, unsigned long request, struct ifreq *ifr)
     strncpy(ifr->ifr_name, name.c_str(), IFNAMSIZ - 1);
 
     if (ioctl(fd, request, (char *)ifr) == -1) {
-        nd_printf("%s: error sending interface request: %s\n",
+        nd_dprintf("%s: error sending interface request: %s\n",
             name.c_str(), strerror(errno));
     }
     else rc = 0;
@@ -1203,4 +1240,93 @@ void ndLogDirectory::Discard(void)
     }
 }
 
+void nd_regex_error(const regex_error &e, string &error)
+{
+    switch (e.code()) {
+    case regex_constants::error_collate:
+        error = "The expression contains an invalid collating element name";
+        break;
+    case regex_constants::error_ctype:
+        error = "The expression contains an invalid character class name";
+        break;
+    case regex_constants::error_escape:
+        error = "The expression contains an invalid escaped character or a trailing escape";
+        break;
+    case regex_constants::error_backref:
+        error = "The expression contains an invalid back reference";
+        break;
+    case regex_constants::error_brack:
+        error = "The expression contains mismatched square brackets ('[' and ']')";
+        break;
+    case regex_constants::error_paren:
+        error = "The expression contains mismatched parentheses ('(' and ')')";
+        break;
+    case regex_constants::error_brace:
+        error = "The expression contains mismatched curly braces ('{' and '}')";
+        break;
+    case regex_constants::error_badbrace:
+        error = "The expression contains an invalid range in a {} expression";
+        break;
+    case regex_constants::error_range:
+        error = "The expression contains an invalid character range (e.g. [b-a])";
+        break;
+    case regex_constants::error_space:
+        error = "There was not enough memory to convert the expression into a finite state machine";
+        break;
+    case regex_constants::error_badrepeat:
+        error = "one of *?+{ was not preceded by a valid regular expression";
+        break;
+    case regex_constants::error_complexity:
+        error = "The complexity of an attempted match exceeded a predefined level";
+        break;
+    case regex_constants::error_stack:
+        error = "There was not enough memory to perform a match";
+        break;
+    default:
+        error = e.what();
+        break;
+    }
+}
+
+bool nd_scan_dotd(const string &path, vector<string> &files)
+{
+    DIR *dh = opendir(path.c_str());
+
+    if (dh == NULL) {
+        nd_printf("Error opening directory: %s: %s\n",
+            path.c_str(), strerror(errno));
+        return false;
+    }
+
+    long dname_max = pathconf(path.c_str(), _PC_NAME_MAX);
+    if (dname_max == -1) dname_max = 255;
+    size_t entry_size = offsetof(struct dirent, d_name) + dname_max + 1;
+
+    struct dirent *dentry = reinterpret_cast<struct dirent *>(
+        new uint8_t[entry_size]
+    );
+    unique_ptr<struct dirent, void(*)(struct dirent *p)> entry(
+        dentry, [](struct dirent *p) {
+        delete [] (uint8_t *)p;
+    });
+
+    int rc;
+    struct dirent *result = NULL;
+    while (! (rc = readdir_r(dh, dentry, &result))) {
+        if (result == NULL) break;
+        if (
+            (
+                result->d_type != DT_LNK &&
+                result->d_type != DT_REG &&
+                result->d_type != DT_UNKNOWN
+            )
+            || ! isdigit(result->d_name[0])) continue;
+        files.push_back(result->d_name);
+    }
+
+    closedir(dh);
+
+    return (rc == 0);
+}
+
 // vi: expandtab shiftwidth=4 softtabstop=4 tabstop=4
